/****************************************************************************
	Read the ReadMe file for instructions.

	This is a part of the NeuroForecaster/GA source code              		
	Copyright (C) 1994-6 NIBS Pte Ltd.  All rights reserved.            		


    PROGRAM: 	DEMO.C

    PURPOSE: 	Set up the data structure (DataStruct) and network structure

	FUNCTION: 	NFGA_INIT()
				NFGA_LOADDATA(LPSTR DataFileName)
				NFGA_LOADNET(LPSTR NetFileName)
				NFGA_FORECAST(NetStruct Net,DataStruct Data)
				NFGA_SAVEFORECAST(LPSTR ResultFileName)
				NFGA_EXIT()

****************************************************************************/
#include <windows.h>
#include <stdio.h>
#include <io.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>
#include <fcntl.h>
#include <ctype.h> 
#include <direct.h> 

#include "demo.h"

FARPROC lpProcAbout;

int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
HANDLE hInstance;
HANDLE hPrevInstance;
LPSTR lpCmdLine;
int nCmdShow;
{

    HWND hWnd;
    MSG msg;

    if (!hPrevInstance)
	if (!DemoInit(hInstance))
	    return (NULL);

    hInst = hInstance;

    hWnd = CreateWindow("Demo",
	"Demo Sample Application",
	WS_OVERLAPPEDWINDOW,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	CW_USEDEFAULT,
	NULL,
	NULL,
	hInstance,
	NULL);

    if (!hWnd)
	return (NULL);

    //ShowWindow(hWnd, nCmdShow);
    ShowWindow(hWnd, SW_SHOWMAXIMIZED);
    UpdateWindow(hWnd);

	lpProcAbout = MakeProcInstance(About, hInst);
	DialogBox(hInst, "AboutBox", hWnd, lpProcAbout);
	FreeProcInstance(lpProcAbout);

    while (GetMessage(&msg, NULL, NULL, NULL)) {
		TranslateMessage(&msg);
		DispatchMessage(&msg);
    }
    return (msg.wParam);
}

/****************************************************************************

    FUNCTION: DemoInit(HANDLE)

    PURPOSE: Initializes window data and registers window class

****************************************************************************/

BOOL DemoInit(hInstance)
HANDLE hInstance;
{
    HANDLE hMemory;
    PWNDCLASS pWndClass;
    BOOL bSuccess;

    hMemory = LocalAlloc(LPTR, sizeof(WNDCLASS));
    pWndClass = (PWNDCLASS) LocalLock(hMemory);
    pWndClass->hCursor = LoadCursor(NULL, IDC_ARROW);
    pWndClass->hIcon = LoadIcon(NULL, IDI_APPLICATION);
    pWndClass->lpszMenuName = (LPSTR) "Menu";
    pWndClass->lpszClassName = (LPSTR) "Demo";
    pWndClass->hbrBackground = GetStockObject(WHITE_BRUSH);
    pWndClass->hInstance = hInstance;
    pWndClass->style = NULL;
    pWndClass->lpfnWndProc = DemoWndProc;

    bSuccess = RegisterClass(pWndClass);

    LocalUnlock(hMemory);
    LocalFree(hMemory);     

    return (bSuccess);
}

/****************************************************************************

    FUNCTION: DemoWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

****************************************************************************/

LONG FAR PASCAL DemoWndProc(hWnd, message, wParam, lParam)
HWND hWnd;
unsigned message;
WORD wParam;
LONG lParam;
{
    HMENU hMenu;

    switch (message) {
		case WM_CREATE:    
    		NFGA_INIT();
    		break;
		case WM_DESTROY:
			NFGA_EXIT();
		    PostQuitMessage(NULL);
		    break;
		case WM_COMMAND:
	    	switch (wParam) {
				case IDM_LoadData:				
					//enter data file name here
					//more data files can be found in the DATA directory of NFGA
					NFGA_LOADDATA("AMETEK.DAT");
		    		break;
				case IDM_LoadNet:             
					//enter network file name
					//more neural network files can be found the FORECAST.ER direcotr of NFGA
					NFGA_LOADNET("AMETEK.GA");
		    		break;
				case IDM_Forecast:
					NFGA_FORECAST();
		    		break;
				case IDM_SaveResult:                
					//enter result file name
					NFGA_SAVEFORECAST("AMETEK.OUT");
		    		break;
				case IDM_Exit:
					NFGA_EXIT();
		    		PostQuitMessage(NULL);
		    		break;

				case IDM_ABOUT:
		    		lpProcAbout = MakeProcInstance(About, hInst);
		    		DialogBox(hInst, "AboutBox", hWnd, lpProcAbout);
		    		FreeProcInstance(lpProcAbout);
		    		break;
				
				//these code are for the drawing functions
				case IDM_BOX:
		    		Shape = SL_BOX;
		    		hMenu = GetMenu(hWnd);
		    		CheckMenuItem(hMenu, IDM_BOX, MF_CHECKED);
		    		CheckMenuItem(hMenu, IDM_BLOCK, MF_UNCHECKED);
		    		break;
				case IDM_BLOCK:
		    		Shape = SL_BLOCK;
		    		hMenu = GetMenu(hWnd);
		    		CheckMenuItem(hMenu, IDM_BOX, MF_UNCHECKED);
		    		CheckMenuItem(hMenu, IDM_BLOCK, MF_CHECKED);
		    		break;

				case IDM_RETAIN:
		    		if (RetainShape) {
						hMenu = GetMenu(hWnd);
						CheckMenuItem(hMenu, IDM_RETAIN, MF_UNCHECKED);
						RetainShape = FALSE;
		    		}
		    		else {
						hMenu = GetMenu(hWnd);
						CheckMenuItem(hMenu, IDM_RETAIN, MF_CHECKED);
						RetainShape = TRUE;
		    		}
		    		break;
	    	}
	    	break;

		case WM_LBUTTONDOWN:     
			// user has pressed the left button
	    	bTrack = TRUE;		 
	    	if (RetainShape)
			SetRectEmpty(&Rect);
	    	StartSelection(hWnd, MAKEPOINT(lParam), &Rect,
			(wParam & MK_SHIFT) ? SL_EXTEND | Shape : Shape);
	    	break;
		case WM_MOUSEMOVE:
		    if (bTrack)
			UpdateSelection(hWnd, MAKEPOINT(lParam), &Rect, Shape);
	    	break;
		case WM_LBUTTONUP:
    	   	if (bTrack)EndSelection(MAKEPOINT(lParam), &Rect);
   	 		bTrack = FALSE;
		    break;
   		case WM_SIZE:
      		switch (wParam) {
         		case SIZEICONIC:
            	// If we aren't in retain mode we want to clear the
	            // current rectangle now!
            	if (!RetainShape)
               SetRectEmpty(&Rect);
      		}
      		break;
		default:
	    	return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return (NULL);
}

/****************************************************************************

    FUNCTION: About(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages for "About" dialog box

****************************************************************************/

BOOL FAR PASCAL About(hDlg, message, wParam, lParam)
HWND hDlg;
unsigned message;
WORD wParam;
LONG lParam;
{
    switch (message) {
	case WM_INITDIALOG:
	    return (TRUE);

	case WM_COMMAND:
	    if (wParam == IDOK
                || wParam == IDCANCEL) {
		EndDialog(hDlg, TRUE);
		return (TRUE);
	    }
	    return (TRUE);
    }
    return (FALSE);
}      
/****************************************************************************

    FUNCTION: StartSelection(HWND, POINT, LPRECT, int)

    PURPOSE: Begin selection of region

****************************************************************************/

int FAR PASCAL StartSelection(HWND hWnd, POINT ptCurrent, LPRECT lpSelectRect, int fFlags)
{
    if (lpSelectRect->left != lpSelectRect->right || lpSelectRect->top != lpSelectRect->bottom)
        ClearSelection(hWnd, lpSelectRect, fFlags);

    lpSelectRect->right = ptCurrent.x;
    lpSelectRect->bottom = ptCurrent.y;

    /* If you are extending the box, then invert the current rectangle */

    if ((fFlags & SL_SPECIAL) == SL_EXTEND)
        ClearSelection(hWnd, lpSelectRect, fFlags);

    /* Otherwise, set origin to current location */

    else
    {
        lpSelectRect->left = ptCurrent.x;
        lpSelectRect->top = ptCurrent.y;
    }
    SetCapture(hWnd);

    return TRUE;
}

/****************************************************************************

    FUNCTION: UpdateSelection(HWND, POINT, LPRECT, int) - update selection area

    PURPOSE: Update selection

****************************************************************************/

int FAR PASCAL UpdateSelection(HWND hWnd, POINT ptCurrent, LPRECT lpSelectRect, int fFlags)
{
    HDC hDC;
    short OldROP;

    hDC = GetDC(hWnd);

    switch (fFlags & SL_TYPE) {

    case SL_BOX:
        OldROP = SetROP2(hDC, R2_NOTXORPEN);
        MoveTo(hDC, lpSelectRect->left, lpSelectRect->top);
        LineTo(hDC, lpSelectRect->right, lpSelectRect->top);
        LineTo(hDC, lpSelectRect->right, lpSelectRect->bottom);
        LineTo(hDC, lpSelectRect->left, lpSelectRect->bottom);
        LineTo(hDC, lpSelectRect->left, lpSelectRect->top);
        LineTo(hDC, ptCurrent.x, lpSelectRect->top);
        LineTo(hDC, ptCurrent.x, ptCurrent.y);
        LineTo(hDC, lpSelectRect->left, ptCurrent.y);
        LineTo(hDC, lpSelectRect->left, lpSelectRect->top);
        SetROP2(hDC, OldROP);
        break;

    case SL_BLOCK:
        PatBlt(hDC,
            lpSelectRect->left,
            lpSelectRect->bottom ,
            lpSelectRect->right - lpSelectRect->left,
            ptCurrent.y - lpSelectRect->bottom,
            DSTINVERT);
        PatBlt(hDC,
            lpSelectRect->right,
            lpSelectRect->top,
            ptCurrent.x - lpSelectRect->right,
            ptCurrent.y - lpSelectRect->top,
            DSTINVERT);
        break;
    }
    lpSelectRect->right = ptCurrent.x;
    lpSelectRect->bottom = ptCurrent.y;
    ReleaseDC(hWnd, hDC);

    return TRUE;
}

/****************************************************************************

    FUNCTION: EndSelection(POINT, LPRECT)

    PURPOSE: End selection of region, release capture of mouse movement

****************************************************************************/

int FAR PASCAL EndSelection(POINT ptCurrent, LPRECT lpSelectRect)
{
    lpSelectRect->right = ptCurrent.x;
    lpSelectRect->bottom = ptCurrent.y;
    ReleaseCapture();

    return TRUE;
}

/****************************************************************************

    FUNCTION: ClearSelection(HWND, LPRECT, int) - clear selection area

    PURPOSE: Clear the current selection

****************************************************************************/

int FAR PASCAL ClearSelection(HWND hWnd, LPRECT lpSelectRect, int fFlags)
{
    HDC hDC;
    short OldROP;

    hDC = GetDC(hWnd);
    switch (fFlags & SL_TYPE) {

    case SL_BOX:
        OldROP = SetROP2(hDC, R2_NOTXORPEN);
        MoveTo(hDC, lpSelectRect->left, lpSelectRect->top);
        LineTo(hDC, lpSelectRect->right, lpSelectRect->top);
        LineTo(hDC, lpSelectRect->right, lpSelectRect->bottom);
        LineTo(hDC, lpSelectRect->left, lpSelectRect->bottom);
        LineTo(hDC, lpSelectRect->left, lpSelectRect->top);
        SetROP2(hDC, OldROP);
        break;

    case SL_BLOCK:
        PatBlt(hDC,
            lpSelectRect->left,
            lpSelectRect->top,
            lpSelectRect->right - lpSelectRect->left,
            lpSelectRect->bottom - lpSelectRect->top,
            DSTINVERT);
        break;
    }
    ReleaseDC(hWnd, hDC);

    return TRUE;
}              

             








